import logging
import os
from glob import glob
from typing import List, Optional

from hexagonal.domain.python_file import PythonFile
from hexagonal.domain.python_project import PythonProject
from hexagonal.services.hexagonal_composition import HexagonalComposition
from hexagonal.services.python_file_builder import PythonFileBuilder


class PythonProjectImporter:
    _composition: HexagonalComposition
    _source_folder_full_path: str

    def import_project(self, *, source_folder: str, composition: HexagonalComposition) -> PythonProject:
        self._composition = composition
        self._source_folder_full_path = os.path.abspath(source_folder)

        python_files = self._import_python_files()

        return PythonProject(full_path=self._source_folder_full_path, python_files=python_files)

    def _get_python_files_in_source_folder(self) -> List[str]:
        return [os.path.abspath(y) for x in os.walk(self._source_folder_full_path)
                for y in glob(os.path.join(x[0], '*.py'))]

    def _import_python_file(self, file: str) -> Optional[PythonFile]:
        logging.info(f'Importing {file}')
        python_file_builder = PythonFileBuilder()
        return python_file_builder.build(project_full_path=self._source_folder_full_path,
                                         file_full_path=file,
                                         composition=self._composition)

    def _import_python_files(self) -> List[PythonFile]:
        valid_files = []
        python_source_files = self._get_python_files_in_source_folder()

        for python_file in python_source_files:
            python_file = self._import_python_file(python_file)
            if python_file.layer_index is None:
                logging.warning(f'File layer index is invalid: {python_file.full_path}')
                continue

            valid_files.append(python_file)

        valid_files.sort(key=lambda valid_file: valid_file.layer_index, reverse=True)
        return valid_files



import typing
from typing import Optional

from hexagonal.domain.hexagonal_layer import HexagonalLayer


class HexagonalComposition(typing.List[HexagonalLayer]):

    def get_layer_index_by_module_name(self, module: str) -> Optional[int]:
        for idx, layer in enumerate(self):
            if module in layer.directories:
                return idx

        return None

    def __add__(self, next_layer: HexagonalLayer) -> 'HexagonalComposition':
        self.clear()
        self.append(next_layer)
        return self

    def __rshift__(self, next_layer: HexagonalLayer) -> 'HexagonalComposition':
        self.append(next_layer)
        return self



from dataclasses import dataclass
from typing import List, Optional

from hexagonal.domain.hexagonal_check_response import HexagonalCheckResponse
from hexagonal.domain.hexagonal_error import HexagonalError
from hexagonal.domain.python_file import PythonFile
from hexagonal.services.hexagonal_composition import HexagonalComposition
from hexagonal.services.python_project_importer import PythonProjectImporter


@dataclass
class SanityCheckResponse:
    FilesChecked: List[PythonFile]
    HexagonalErrors: List[HexagonalError]


class CheckProjectSanityUseCase:
    _hexa_modules_dirs_names = List[str]
    _source_folder_full_path: str
    _source_folder: str
    _composition: HexagonalComposition

    def check(self, *, composition: HexagonalComposition, source_folder: str = '') -> HexagonalCheckResponse:
        importer = PythonProjectImporter()
        python_project = importer.import_project(source_folder=source_folder, composition=composition)

        errors = []
        for python_file in python_project.python_files:
            error = self._check_dependencies_order(python_file=python_file)
            if error:
                errors.append(error)

        result = HexagonalCheckResponse()
        result.errors = errors
        result.project = python_project

        return result

    @staticmethod
    def _check_dependencies_order(python_file: PythonFile) -> Optional[HexagonalError]:
        for imported_module in python_file.imported_modules:
            if python_file.layer_index > imported_module.layer_index:
                return HexagonalError(message='Wrong dependency flow. An inner layer is pointing to an outer layer.',
                                      outer_layer_name=imported_module.layer_name,
                                      inner_layer_name=python_file.layer_name,
                                      python_file_problem=python_file.relative_path_from_source_module,
                                      imported_module_problem=imported_module.module)

        return None



from typing import List

from diagrams import Diagram, Cluster
from diagrams.aws.storage import Storage

from hexagonal.domain.hexagonal_layer import HexagonalLayer


class GenerateDiagramUseCase:

    @staticmethod
    def execute(*, project_name: str, hexagonal_composition: List[HexagonalLayer], show: bool = False,
                output_file_name: str = '') -> bool:
        last_node = None
        with Diagram(project_name, show=show, filename=output_file_name):
            for layer in hexagonal_composition:
                with Cluster(layer.name):
                    is_first = True
                    for module in layer.directories:
                        cur_node = Storage(label=module)
                        if is_first:
                            is_first = False
                            if last_node is not None:
                                last_node >> cur_node

                        last_node = cur_node

        return last_node is not None



import logging
import os
import sys
from runpy import run_path

import click

from hexagonal.domain.hexagonal_error import HexagonalError
from hexagonal.main import hexagonal_config
from hexagonal.use_cases.check_project_sanity_usecase import CheckProjectSanityUseCase
from hexagonal.use_cases.generate_diagram_usecase import GenerateDiagramUseCase


@click.group(help='Options for Hexagonal Sanity Check')
def cli():
    pass


@click.command()
@click.option('--source_path', help='Where main source folder is located.', required=True)
@click.option('--hexagonal_config_file', default='hexagonal_config.py', help="Hexagonal configuration file's name.")
def diagram(source_path, hexagonal_config_file):
    _process_cli_arguments(source_path=source_path, hexagonal_config_file=hexagonal_config_file)

    try:
        hexa_diagram = GenerateDiagramUseCase()
        hexa_diagram.execute(project_name='Hexagonal Architecture Diagram', hexagonal_composition=hexagonal_config,
                             show=True)
    except Exception as error:
        click.echo(f'Error while generating project\'s diagram: "{error}"')
        exit(1)


@click.command()
@click.option('--source_path', help='Where main source folder is located.', required=True)
@click.option('--hexagonal_config_file', default='hexagonal_config.py', help="Hexagonal configuration file's name.")
def check(source_path, hexagonal_config_file):
    _process_cli_arguments(source_path=source_path, hexagonal_config_file=hexagonal_config_file)

    try:
        checker = CheckProjectSanityUseCase()
        response = checker.check(composition=hexagonal_config, source_folder=source_path)
    except Exception as error:
        click.echo(f'Error while processing project: "{error}"')
        exit(1)

    [_print_error(index, error) for index, error in enumerate(response.errors)]
    if len(response.errors) > 0:
        logging.error('Hexagonal Architecture: Errors found in dependencies flow.')
        exit(1)

    click.echo('Hexagonal Architecture: No errors found.')


def _print_error(error_index: int, error: HexagonalError):
    if error_index == 0:
        click.echo('')
        click.echo('#' * 100)
        click.echo('## Hexagonal Architecture errors found:')
        click.echo('## First fix the [ERROR 1] and re-run the check. '
                   'Follows errors can be cascading from it. \n')

    click.echo(f'[ERROR {error_index + 1}] Hexagonal Architecture: {error.message}')
    click.echo(f'    Wrong flow: {error.inner_layer_name} -> {error.outer_layer_name}')
    click.echo(f'    Python file: {error.python_file_problem}')
    click.echo(f'    Outer Module : {error.imported_module_problem}')


def _validate_project_source_path(source_path: str):
    click.echo(f'Checking project at: {source_path}')
    if not os.path.isdir(source_path):
        click.echo('Project folder not found.')
        exit(1)
    sys.path.append(source_path)


def _import_hexagonal_config_file(hexagonal_config_file: str):
    click.echo(f'Checking hexagonal configuration file at: {hexagonal_config_file}')
    if not os.path.isfile(hexagonal_config_file):
        click.echo('Project configuration file not found.')
        exit(1)
    hexagonal_config.clear()
    run_path(hexagonal_config_file)


def _process_cli_arguments(source_path: str, hexagonal_config_file: str):
    source_path = os.path.abspath(source_path)
    hexagonal_config_file = source_path + '/' + hexagonal_config_file

    _validate_project_source_path(source_path=source_path)
    _import_hexagonal_config_file(hexagonal_config_file=hexagonal_config_file)


cli.add_command(check)
cli.add_command(diagram)


def main():
    logging.basicConfig(format='%(asctime)s %(levelname)s: %(message)s', stream=sys.stdout, level=logging.ERROR)
    cli()


if __name__ == '__main__':
    main()



from dataclasses import dataclass
from typing import Optional, List

from hexagonal.domain.python_module import PythonModule


@dataclass
class PythonFile:
    full_path: str
    relative_path_from_source_module: str
    layer_name: str
    layer_index: Optional[int]
    imported_modules: List[PythonModule]



from typing import List

from hexagonal.domain.hexagonal_error import HexagonalError
from hexagonal.domain.python_project import PythonProject


class HexagonalCheckResponse:
    errors: List[HexagonalError]
    project: PythonProject



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['hexagonal',
 'hexagonal.domain',
 'hexagonal.infrastructure',
 'hexagonal.services',
 'hexagonal.use_cases']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.0.3,<9.0.0', 'diagrams>=0.20.0,<0.21.0']

entry_points = \
{'console_scripts': ['hexagonal = hexagonal.infrastructure.cli:main']}

setup_kwargs = {
    'name': 'hexagonal-sanity-check',
    'version': '0.0.15',
    'description': 'Hexagonal Sanity Check',
    'long_description': "# Hexagonal Sanity Check\n\nThis project checks if the dependency flow between the layers of the Hexagonal architecture defined \nfor this project was respected.\n\n### How to configure\n\nFirst it's necessary to define your hexagonal layers and their order.\nThe tool expects a default file name on your source folder dir with the name `hexagonal_config.py`.\n\n1. First you create the Hexagonal Layers you have on your system via the class name HexagonalLayer.\nThere are two arguments: \n   - `name`: It can be any name, `domain`, `frontend`, `infrastructure`, or any name you used for your layers.\n   - `usecases`: This is the name of the directory the files related to this layer as storage. It's not the full path, \njust the directory name.\n\n2. Import `hexagonal_config` on your file, and define the order with `+` (add layers)\nthen `>>`(set the sequence of the layers). The most to the left layers is the most outer layer, while\nthe most to the right layer is the most inner layer.\n\nExample, for this folder structure:\n```\n. src\n├── __init__.py\n├── hexagonal_config.py\n├── domain\n│\xa0\xa0 ├── __init__.py\n│\xa0\xa0 ├── __pycache__\n│\xa0\xa0 └── person.py\n├── infrastructure\n│\xa0\xa0 ├── __init__.py\n│\xa0\xa0 └── person_mysql_repository.py\n├── main.py\n├── services\n│\xa0\xa0 ├── __init__.py\n│\xa0\xa0 └── person_repository.py\n└── usecases\n    ├── __init__.py\n    └── create_person_usecase.py\n```\nThe file:\n```python\nfrom hexagonal.domain.hexagonal_layer import HexagonalLayer\nfrom hexagonal.main import hexagonal_config\n\ninfrastructure_layer = HexagonalLayer(name='infrastructure', directories=['infrastructure'])\nuse_cases_layer = HexagonalLayer(name='use_cases', directories=['usecases'])\nservices_layer = HexagonalLayer(name='services', directories=['services'])\ndomain_layer = HexagonalLayer(name='domain', directories=['domain'])\n\nhexagonal_config + infrastructure_layer >> use_cases_layer >> services_layer >> domain_layer\n```\n\n\n### Generating the Project Diagram\nThis command generate a visual diagram show the composition of your hexagonal layers.\n\n#### Pre requisites\nTo generate the Hexagonal Diagram of the project, it's necessary to have Graphviz installed in the machine.  \nFor Mac you can ``brew install graphviz``.  \nFor other, check the documentation https://graphviz.org/download/. \n\n#### CMD\n`hexagonal diagram --source_path ./` \n\n### Checking Project's Hexagonal Integrity \nThis checks if the correct flow of the dependencies -from outer to inner layer- was respected.\n\n#### CMD\n`hexagonal check --source_path ./`\n\n",
    'author': 'rfrezino',
    'author_email': 'rodrigofrezino@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
